def linearspline(time,data,fixedpoint=None):
  """Linear spline calculation.
This function fits a linear spline function to a given set of
time-course gene expression measurements. It takes three arguments:
time:        A list containing the time points at which measurements
             were made.
data:        A list of lists, each list being the data measured at a
             time point.
fixedpoint:  If fixedpoint is set equal to a number, then the linear spline
             function is forced to pass through this value at the first 
             time point (i.e., the first element of time).
             This is useful if the gene expression data are given as log-
             ratios with respect to time zero, in which case the log-ratio
             is equal to zero by definition at time zero.
             If fixedpoint is given, the first element of data should
             be an empty list.
This function returns the tuple (knots, f, error), in which
knots:   The indices of the time points at which there is a knot
f:       The value of the linear spline function at the knots
error:   The total squared error between the linear spline function
         and the data set to which the linear spline function was fitted,
         divided by the total square error if a flat line is fitted to
         the data (i.e., the relative error). This is equal to
         1.-(fraction variance explained). 
The usage of linear spline function in gene expression data analysis is
described in:
M.J.L. de Hoon; S. Imoto; S. Miyano: "Statistical analysis of a small
set of time-ordered gene expression data using linear splines".
Bioinformatics, 18: 1477--1485 (2002).
"""
  from linspline import linspline
  from Numeric import concatenate
  if type(time)!=type([]):
    # user passed an array instead of a list
    time = time.tolist()
  n = len(time)
  ndata = [len(list) for list in data]
  if fixedpoint==None:
    lfixed = 0
    fixed = 0.
  else:
    lfixed = 1
    fixed = fixedpoint
  nknots, knots, f, error, info = linspline(time,ndata,concatenate(data),lfixed,fixed)
  if info:
    print "Error calculating linear spline function:"
    print "Not all time points have measured data"
    return
  else:
    knots = knots.tolist()
    knots = knots[:nknots]
    f = f[:nknots]
    return knots,f,error
